@@ -763,6 +763,9 @@ static int cleanup_fds(void)
 #endif
 #ifdef EMFILE
 	if (errno == EMFILE) return abort_background_connections();
+#endif
+#ifndef EMFILE
+	if (errno == EIO) return abort_background_connections();
 #endif
 	return 0;
 }
@@ -777,6 +780,16 @@ int c_pipe(int fd[2])
 		EINTRLOOP(r, pipe(fd));
 		if (!r) new_fd_bin(fd[0]), new_fd_bin(fd[1]);
 		fd_unlock();
+		if (verify_handle(fd[0]) || verify_handle(fd[1])) {
+			EINTRLOOP(r, close(fd[0]));
+			EINTRLOOP(r, close(fd[1]));
+			r = -1;
+#ifdef EMFILE
+			errno = EMFILE;
+#else
+			errno = EIO;
+#endif
+		}
 	} while (r == -1 && cleanup_fds());
 	return r;
 }
@@ -789,6 +802,16 @@ int c_dup(int oh)
 		EINTRLOOP(h, dup(oh));
 		if (h != -1) new_fd_cloexec(h);
 		fd_unlock();
+		if (verify_handle(h)) {
+			int r;
+			EINTRLOOP(r, close(h));
+			h = -1;
+#ifdef EMFILE
+			errno = EMFILE;
+#else
+			errno = EIO;
+#endif
+		}
 	} while (h == -1 && cleanup_fds());
 	return h;
 }
@@ -801,6 +824,16 @@ int c_socket(int d, int t, int p)
 		EINTRLOOP(h, socket(d, t, p));
 		if (h != -1) new_fd_cloexec(h);
 		fd_unlock();
+		if (verify_handle(h)) {
+			int r;
+			EINTRLOOP(r, close(h));
+			h = -1;
+#ifdef EMFILE
+			errno = EMFILE;
+#else
+			errno = EIO;
+#endif
+		}
 	} while (h == -1 && cleanup_fds());
 	return h;
 }
@@ -813,6 +846,16 @@ int c_accept(int sh, struct sockaddr *addr, socklen_t *addrlen)
 		EINTRLOOP(h, accept(sh, addr, addrlen));
 		if (h != -1) new_fd_cloexec(h);
 		fd_unlock();
+		if (verify_handle(h)) {
+			int r;
+			EINTRLOOP(r, close(h));
+			h = -1;
+#ifdef EMFILE
+			errno = EMFILE;
+#else
+			errno = EIO;
+#endif
+		}
 	} while (h == -1 && cleanup_fds());
 	return h;
 }
@@ -825,6 +868,16 @@ int c_open(unsigned char *path, int flags)
 		EINTRLOOP(h, open(cast_const_char path, flags));
 		if (h != -1) new_fd_bin(h);
 		fd_unlock();
+		if (verify_handle(h)) {
+			int r;
+			EINTRLOOP(r, close(h));
+			h = -1;
+#ifdef EMFILE
+			errno = EMFILE;
+#else
+			errno = EIO;
+#endif
+		}
 	} while (h == -1 && cleanup_fds());
 	return h;
 }
@@ -837,6 +890,16 @@ int c_open3(unsigned char *path, int flags, int mode)
 		EINTRLOOP(h, open(cast_const_char path, flags, mode));
 		if (h != -1) new_fd_bin(h);
 		fd_unlock();
+		if (verify_handle(h)) {
+			int r;
+			EINTRLOOP(r, close(h));
+			h = -1;
+#ifdef EMFILE
+			errno = EMFILE;
+#else
+			errno = EIO;
+#endif
+		}
 	} while (h == -1 && cleanup_fds());
 	return h;
 }
@@ -2923,7 +2986,7 @@ void unhandle_mouse(void *data) { }
 #endif /* #ifdef USE_GPM */
 
 
-#if defined(WIN) || defined(INTERIX)
+#if defined(OS2) || defined(WIN) || defined(INTERIX)
 
 static int is_remote_connection(void)
 {
@@ -3628,16 +3691,20 @@ void restore_terminal(void)
 void os_report_error_va(const char *caption, const char *msg, va_list l)
 {
 #ifdef OS2
-	char msg_buffer[OS_REPORT_ERROR_BUFFER];
-	vsnprintf(msg_buffer, sizeof msg_buffer, msg, l);
-	if (os2_init_pm()) return;
-	WinMessageBox(HWND_DESKTOP, NULLHANDLE, msg_buffer, caption, 0, MB_CANCEL | MB_ERROR | MB_APPLMODAL | MB_MOVEABLE);
-	os2_exit_pm();
+	if (!is_remote_connection()) {
+		char msg_buffer[OS_REPORT_ERROR_BUFFER];
+		vsnprintf(msg_buffer, sizeof msg_buffer, msg, l);
+		if (os2_init_pm()) return;
+		WinMessageBox(HWND_DESKTOP, NULLHANDLE, msg_buffer, caption, 0, MB_CANCEL | MB_ERROR | MB_APPLMODAL | MB_MOVEABLE);
+		os2_exit_pm();
+	}
 #endif
 #if defined(WIN) && !(defined(_UWIN) && !defined(GRDRV_PMSHELL))
-	char msg_buffer[OS_REPORT_ERROR_BUFFER];
-	vsnprintf(msg_buffer, sizeof msg_buffer, msg, l);
-	MessageBoxA(NULL, msg_buffer, caption, MB_OK | MB_ICONEXCLAMATION);
+	if (!is_remote_connection()) {
+		char msg_buffer[OS_REPORT_ERROR_BUFFER];
+		vsnprintf(msg_buffer, sizeof msg_buffer, msg, l);
+		MessageBoxA(NULL, msg_buffer, caption, MB_OK | MB_ICONEXCLAMATION);
+	}
 #endif
 }
 
